from quixote import get_response
from quixote.html import htmltext
from quixote.directory import Directory, AccessControlled

from wcs.workflows import *
from qommon.errors import *

from wcs.forms.common import FormStatusPage

def form_attachment(self):
    self.check_receiver()

    try:
        fn = get_request().form['f']
    except (KeyError, ValueError):
        raise TraversalError()

    found = False
    for evo in self.filled.evolution:
        if evo.parts:
            for p in evo.parts:
                if not isinstance(p, AttachmentEvolutionPart):
                    continue
                if os.path.basename(p.filename) == fn:
                    found = True
                    break
            if found:
                break

    if not found:
        raise TraversalError()

    try:
        fd = file(p.filename)
    except:
        raise TraversalError()
    content = fd.read()
    fd.close()

    response = get_response()
    if p.content_type:
        response.set_content_type(p.content_type)
    else:
        response.set_content_type('application/octet-stream')
    if p.charset:
        response.set_charset(p.charset)
    if p.base_filename:
        if p.content_type and p.content_type.startswith('image/'):
            response.set_header(
                'content-disposition', 'inline; filename="%s"' % p.base_filename)
        else:
            response.set_header(
                'content-disposition', 'attachment; filename="%s"' % p.base_filename)

    return content




class AttachmentEvolutionPart:
    orig_filename = None
    base_filename = None
    content_type = None
    charset = None

    def __init__(self, f):
        self.orig_filename = f.orig_filename
        self.base_filename = f.base_filename
        self.content_type = f.content_type
        self.charset = f.charset
        self.fp = f.fp

    def __getstate__(self):
        odict = self.__dict__.copy()
        if not odict.has_key('fp'):
            return odict

        del odict['fp']
        dirname = os.path.join(get_publisher().app_dir, 'attachments')
        if not os.path.exists(dirname):
            os.mkdir(dirname)
        odict['filename'] = os.path.join(dirname,
                str(max([lax_int(x) for x in [os.listdir(dirname)] + [0] ])))
        self.fp.seek(0)
        fd = file(odict['filename'], 'w')
        fd.write(self.fp.read())
        fd.close()
        return odict

    def view(self):
        return htmltext('<p><a href="attachment?f=%s">%s</a>' % (
                    os.path.basename(self.filename), self.orig_filename))

class AddAttachmentWorkflowStatusItem(WorkflowStatusItem):
    description = N_('Add Attachment')
    key = 'addattachment'

    by = None

    def init(cls):
        FormStatusPage._q_extra_exports.append('attachment')
        FormStatusPage.attachment = form_attachment
    init = classmethod(init)

    def fill_form(self, form, formdata, user):
        form.add(FileWidget, 'attachment', title = _('Upload File'), required = True)
        form.add_submit(self.id, _('Upload File'))

    def submit_form(self, form, formdata, user, evo):
        if form.get_submit() == self.id:
            f = form.get_widget('attachment').parse()
            if f is None:
                form.set_error('attachment', _('Missing file'))
                return
            evo.add_part(AttachmentEvolutionPart(f))

    def fill_admin_form(self, form):
        form.add(WidgetList, 'by', title = _('By'), element_type = SingleSelectWidget,
                value = self.by,
                add_element_label = _('Add Role'),
                element_kwargs = {'render_br': False,
                'options': [(None, '---'),
                            ('_submitter', _('Sender')),
                            ('_receiver', _('Receiver')),
                            (logged_users_role().id, logged_users_role().name),
                            (None, '----')] + get_user_roles()})


    def submit_admin_form(self, form):
        for f in ('by',):
            widget = form.get_widget(f)
            if widget:
                setattr(self, f, widget.parse())

register_item_class(AddAttachmentWorkflowStatusItem)
