from quixote import get_publisher

import os
import signal
import time
try:
    import dl
except ImportError:
    dl = None
import sys

from storage import StorableObject

parent_killed = False

def stop_cron_process(*args):
    global parent_killed
    parent_killed = True

class CronJob:
    hours = None
    minutes = None
    weekdays = None
    days = None
    function = None

    def __init__(self, function, hours = None, minutes = None, weekdays = None, days = None):
        self.function = function
        self.hours = hours
        self.minutes = minutes
        self.weekdays = weekdays
        self.days = days


def cron(publisher):
    global parent_killed
    app_dir = publisher.app_dir

    last = time.localtime()
    while not parent_killed:
        now = time.localtime()
        if now[:5] != last[:5]:
            last = now
            for hostname in os.listdir(app_dir):
                publisher.app_dir = os.path.join(app_dir, hostname)
                if not os.path.isdir(publisher.app_dir):
                    continue
                for job in publisher.cronjobs:
                    if job.days and now[2] not in job.days:
                        continue
                    if job.weekdays and now[6] not in job.weekdays:
                        continue
                    if job.hours and not now[3] in job.hours:
                        continue
                    if job.minutes and not now[4] in job.minutes:
                        continue
                    job.function(publisher)
        time.sleep(10)


def spawn_cron(create_publisher):
    global cron_pid
    cron_pid = os.fork()
    if cron_pid == 0:
        pub = create_publisher()
        # set process name, this only works on Linux, 15 == PR_SET_NAME
        if dl:
            libc = dl.open('/lib/libc.so.6')
            libc.call('prctl', 15, os.path.basename(sys.argv[0]) + ' [cron]\0', 0, 0, 0)
        signal.signal(signal.SIGTERM, stop_cron_process)
        cron(pub)
        sys.exit(0)
    else:
        signal.signal(signal.SIGTERM, kill_cron_process)
        #os.close(parent_fd)

def kill_cron_process(signum, frame):
    os.kill(cron_pid, signal.SIGTERM)
    sys.exit(0)

