import sys
from email.MIMEText import MIMEText
from email.MIMEMultipart import MIMEMultipart
from email.Header import Header
import smtplib
import socket
from vendor import timeoutsocket

from cStringIO import StringIO
import ezt

try:
    import docutils
    import docutils.core
    import docutils.io
except ImportError:
    docutils = None

from quixote import get_request, get_response, get_publisher

from publisher import get_cfg, get_logger
import errors
import tokens
from admin.emails import EmailsDirectory


class SMTPWithTimeout(smtplib.SMTP):
    timeout = None

    def __init__(self, host = '', port = 0, local_hostname = None, timeout = None):
        self.timeout = timeout
        smtplib.SMTP.__init__(self, host, port, local_hostname)

    def connect(self, host='localhost', port = 0):
        '''Connect to a host on a given port.
        Override the standard SMTP connect in order to use a timeoutsocket.
        '''
        if not port:
            i = host.rfind(':')
            if i >= 0:
                host, port = host[:i], host[i+1:]
                try:
                    port = int(port)
                except ValueError:
                    raise socket.error, 'nonnumeric port'
        if not port:
            port = smtplib.SMTP_PORT
        if self.debuglevel > 0:
            print >> sys.stderr, 'connect:', (host, port)
        msg = 'getaddrinfo returns an empty list'
        self.sock = None
        for res in socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM):
            af, socktype, proto, canonname, sa = res
            try:
                self.sock = timeoutsocket.TimeoutSocket(
                        socket.socket(af, socktype, proto), self.timeout)
                if self.debuglevel > 0:
                    print >> sys.stderr, 'connect:', sa
                self.sock.connect(sa)
            except socket.error, msg:
                if self.debuglevel > 0:
                    print >> sys.stderr, 'connect fail:', msg
                if self.sock:
                    self.sock.close()
                self.sock = None
                continue
            except timeoutsocket.Timeout:
                if self.debuglevel > 0:
                    print >> sys.stderr, 'connect failed (timeout)'
                if self.sock:
                    self.sock.close()
                self.sock = None
                msg = 'connect timeout'
            break
        if not self.sock:
            if msg == 'connect timeout':
                raise timeoutsocket.Timeout()
            raise socket.error, msg
        code, msg = self.getreply()
        if self.debuglevel >0 :
            print >> sys.stderr, 'connect:', msg
        return (code, msg)


def custom_ezt_email(key, mail_body_data, email_rcpt, **kwargs):

    if not EmailsDirectory.is_enabled(key):
        return

    mail_subject = EmailsDirectory.get_subject(key)
    mail_body = EmailsDirectory.get_body(key)

    if not mail_body_data:
        mail_body_data = {}
    if not mail_body_data.get('sitename'):
        mail_body_data['sitename'] = get_cfg('misc', {}).get('sitename')

    return ezt_email(mail_subject, mail_body, mail_body_data, email_type = key,
            email_rcpt = email_rcpt, **kwargs)


def ezt_email(subject, mail_body, mail_body_data, email_rcpt, email_type = None, **kwargs):

    mail_subject_template = ezt.Template(compress_whitespace = False)
    mail_subject_template.parse(subject)

    fd = StringIO()
    mail_subject_template.generate(fd, mail_body_data)
    real_mail_subject = fd.getvalue()

    mail_body_template = ezt.Template(compress_whitespace = False)
    mail_body_template.parse(mail_body)

    fd = StringIO()
    mail_body_template.generate(fd, mail_body_data)
    real_mail_body = fd.getvalue()

    return email(real_mail_subject, real_mail_body, email_rcpt = email_rcpt,
            email_type = email_type, **kwargs)


def email(subject, mail_body, email_rcpt, replyto = None, bcc = None,
        exclude_current_user = False, email_type = None, want_html = True,
        hide_recipients = False, fire_and_forget = False,
        smtp_timeout = None):
    emails_cfg = get_cfg('emails', {})

    encoding = get_publisher().site_charset

    mail_body = str(mail_body)
    htmlmail = None
    if want_html:
        try:
            htmlmail, pub = docutils.core.publish_programmatically(
                    source_class = docutils.io.StringInput,
                    source = mail_body,
                    source_path = None,
                    destination_class = docutils.io.StringOutput,
                    destination = None,
                    destination_path = None,
                    reader = None, reader_name = 'standalone',
                    parser = None, parser_name = 'restructuredtext',
                    writer = None, writer_name = 'html',
                    settings = None, settings_spec = None,
                    settings_overrides = {'input_encoding': encoding,
                        'output_encoding': encoding,
                        'embed_stylesheet': False,
                        'stylesheet': None,
                        'stylesheet_path': None,
                        'file_insertion_enabled': 0,
                        'xml_declaration': 0,
                        'report_level': 5},
                    config_section = None,
                    enable_exit_status = None)
        except:
            pass
        else:
            if pub.document.reporter.max_level >= 3:
                # notify user
                pass

    if mail_body[0] == '<':
        htmlmail = mail_body
        mail_body = None

    msg = MIMEMultipart(_charset = encoding, _subtype = 'alternative')
    msg['Subject'] = Header(subject, encoding)
    if hide_recipients:
        msg['To'] = 'Undisclosed recipients:;'
    else:
        if type(email_rcpt) is list:
            msg['To'] = ', '.join(email_rcpt)
        else:
            msg['To'] = email_rcpt
    msg['From'] = emails_cfg.get('from', 'noreply@entrouvert.com')
    if emails_cfg.get('reply_to'):
        msg['Reply-To'] = emails_cfg.get('reply_to')
    if replyto:
        msg['Reply-To'] = replyto

    msg.preamble = ''
    msg.epilogue = ''

    if mail_body:
        msg.attach(MIMEText(mail_body, _charset = encoding))

    if htmlmail:
        msg.attach(MIMEText(htmlmail, _subtype = 'html', _charset = encoding))

    if type(email_rcpt) is list:
        rcpts = email_rcpt[:]
    else:
        rcpts = [email_rcpt]
    if bcc:
        rcpts += bcc

    if exclude_current_user:
        user = get_request().user
        if user and user.email and user.email in rcpts:
            rcpts.remove(user.email)

    rcpts = [x for x in rcpts if x]
    if len(rcpts) == 0:
        return

    if not fire_and_forget:
        try:
            s = SMTPWithTimeout(emails_cfg.get('smtp_server', None) or 'localhost',
                    timeout = smtp_timeout)
        except timeoutsocket.Timeout:
            get_logger().error('Failed to connect to SMTP server (timeout)')
            raise errors.EmailError()
        except socket.error:
            # XXX: write message in a queue somewhere?
            get_logger().error('Failed to connect to SMTP server')
            raise errors.EmailError()
        if not s.sock:
            get_logger().error('Failed to connect to SMTP server')
            raise errors.EmailError()

    if emails_cfg.get('bounce_handler'):
        if get_request():
            server_name = get_request().environ.get('SERVER_NAME').strip()
        else:
            server_name = msg['From'].split('@')[1]
        token = tokens.Token(7 * 86400)
        token.type = 'email-bounce'
        token.email_rcpts = [str(x) for x in rcpts]
        token.email_message = msg.as_string()
        token.email_type = str(email_type)
        token.store()
        msg_from = '%s-bounces+%s@%s' % (get_publisher().APP_NAME, token.id, server_name)
    else:
        msg_from = msg['From']

    if not fire_and_forget:
        try:
            s.sendmail(msg_from, rcpts, msg.as_string())
        except smtplib.SMTPRecipientsRefused:
            get_logger().error('Failed to send mail to %s' % ', '.join(rcpts))
        s.close()
    else:
        get_response().add_after_job('sending email',
                EmailToSend(msg_from, rcpts, msg.as_string()),
                fire_and_forget = True)

class EmailToSend:
    def __init__(self, msg_from, rcpts, msg_as_string):
        self.msg_from = msg_from
        self.rcpts = rcpts
        self.msg_as_string = msg_as_string

    def __call__(self):
        emails_cfg = get_cfg('emails', {})

        try:
            s = smtplib.SMTP(emails_cfg.get('smtp_server', None) or 'localhost')
        except socket.error:
            return
        if not s.sock:
            return

        try:
            s.sendmail(self.msg_from, self.rcpts, self.msg_as_string)
        except smtplib.SMTPRecipientsRefused:
            pass
        s.close()
