import base64
import cStringIO
import mimetypes
import os
import re
import socket
import tempfile
import time
import random
import datetime

try:
    from vendor.feedparser import _sanitizeHTML
except ImportError:
    _sanitizeHTML = None

try:
    import DNS
    DNS.ParseResolvConf()
except ImportError:
    DNS = None

import quixote

from quixote import get_publisher, get_request, get_response, get_session
from quixote.http_request import Upload
from quixote.http_response import status_reasons
from quixote.form import *
from quixote.html import htmltext, htmltag, htmlescape, TemplateIO
from quixote.util import randbytes

import misc
from strftime import strftime
from publisher import get_cfg

QuixoteForm = Form

Widget.REQUIRED_ERROR = N_('required field')
get_error_orig = Widget.get_error

def get_i18n_error(self, request=None):
    error = get_error_orig(self, request)
    if error == Widget.REQUIRED_ERROR:
        return _(error)
    return error

def render(self):
    # quixote/form/widget.py, Widget::render
    r = TemplateIO(html=True)
    classnames = '%s widget' % self.__class__.__name__
    if hasattr(self, 'extra_css_class') and self.extra_css_class:
        classnames += ' ' + self.extra_css_class
    if self.get_error():
        classnames += ' widget-with-error'
    r += htmltext('<div class="%s">') % classnames
    r += self.render_title(self.get_title())
    r += htmltext('<div class="content">')
    r += self.render_content()
    r += self.render_hint(self.get_hint())
    r += self.render_error(self.get_error())
    r += htmltext('</div>')
    r += htmltext('</div>')
    if self.render_br:
        r += htmltext('<br class="%s" />') % classnames
    r += htmltext('\n')
    return r.getvalue()

Widget.get_error = get_i18n_error
Widget.render = render
Widget.cleanup = None

class Form(QuixoteForm):
    TOKEN_NOTICE = N_(
            "The form you have submitted is invalid.  Most "
            "likely it has been successfully submitted once "
            "already.  Please review the form data "
            "and submit the form again.")
    ERROR_NOTICE = N_("There were errors processing your form.  See below for details.")

    info = None
    captcha = None

    def __init__(self, *args, **kwargs):
        QuixoteForm.__init__(self, *args, **kwargs)

    def add_captcha(self):
        if not self.captcha and not (get_session().won_captcha or get_session().user):
            self.captcha = CaptchaWidget('captcha')

    def add(self, widget_class, name, *args, **kwargs):
        if kwargs and not kwargs.has_key('render_br'):
            kwargs['render_br'] = False
        return QuixoteForm.add(self, widget_class, name, *args, **kwargs)

    def get_all_widgets(self):
        l = QuixoteForm.get_all_widgets(self)
        if self.captcha:
            l.append(self.captcha)
        return l

    def _get_default_action(self):
        return get_request().get_path()

    def _render_submit_widgets(self):
        return htmltext('<div class="buttons">%s</div>' % QuixoteForm._render_submit_widgets(self))

    def _render_info_notice(self):
        return htmltext('<div class="infonotice">%s</div>' % _(self.info))

    def _render_error_notice(self):
        return htmltext('<div class="errornotice">%s</div>' % _(
                    QuixoteForm._render_error_notice(self)))

    def _render_body(self):
        r = TemplateIO(html=True)
        if self.has_errors():
            r += self._render_error_notice()
        if self.info:
            r += self._render_info_notice()
        r += self._render_widgets()
        if self.captcha:
            r += self.captcha.render()
        r += self._render_submit_widgets()
        if self.has_errors():
            # if form has errors, focus and select the first widget on error
            r += htmltext("""<script type="text/javascript">
var all_obj = document.getElementsByTagName('DIV');
for (i=0; i<all_obj.length; i++) {
    if ( ( ' ' + all_obj[i].getAttribute("class") + ' ').toLowerCase().match(
                new RegExp ( '^.* widget-with-error .*$', 'g' ) ) ) {
        less_obj = all_obj[i].getElementsByTagName('INPUT');
        for (j=0; j<less_obj.length; j++) {
            less_obj[j].select();
            less_obj[j].focus();
            break;
        }
        break;
    }
}
</script>""")
        return r.getvalue()


class HtmlWidget:
    error = None

    def __init__(self, string, title = None, *args, **kwargs):
        self.string = string
        self.title = title

    def render(self):
        return self.string

    def render_content(self):
        if self.title:
            return self.title
        return self.string

    def has_error(self, request):
        return False

    def parse(self, *args):
        pass


class FileWithPreviewWidget(CompositeWidget):
    """Widget that proposes a File Upload widget but that stores the file
    ondisk so it has a "readonly" mode where the filename is shown."""
    token = None

    def __init__(self, name, value=None, **kwargs):
        CompositeWidget.__init__(self, name, value, **kwargs)
        self.value = value
        preview = kwargs.get('readonly')
        if not preview:
            del kwargs['title']
            if kwargs.has_key('hint'):
                del kwargs['hint']
            self.add(FileWidget, 'file', **kwargs)
        else:
            session = get_session()
            self.add(HiddenWidget, 'token')
            del kwargs['title']
            self.add(StringWidget, 'filename', **kwargs)
            if self.value and self.value.orig_filename:
                    wid = self.get_widget('filename')
                    get_request().form[wid.name] = self.value.orig_filename

            # Is this still mandatory ?
            if hasattr(get_request(), 'tempfiles'):
                if name in get_request().tempfiles:
                    token = get_request().tempfiles[name]
                    wid = self.get_widget('token')
                    get_request().form[wid.name] = token

    def render_content(self):
        r = TemplateIO(html=True)
        if isinstance(self.value, PicklableUpload):
            filetype = mimetypes.guess_type(self.value.orig_filename)
            if filetype and filetype[0] and filetype[0].startswith("image"):
                session = get_session()
                if self.token in session.tempfiles.keys():
                    session.tempfiles[self.token]['value'] = self.value.fp.read()
                    r += htmltext('<img alt="%s" src="tempfile?t=%s" />' % \
                                            (self.value.orig_filename,
                                             self.token))
                    self.value.fp.seek(0)
        for widget in self.get_widgets():
            r += widget.render()
        return r.getvalue()

    def _parse(self, request):
        session = get_session()
        if self.get('file'):
            self.value = self.get('file')
            self.value.__class__ = PicklableUpload
            # and saves it
            token = randbytes(8)
            if not hasattr(get_request(), 'tempfiles'):
                get_request().tempfiles = {}
            get_request().tempfiles[self.name] = token
            dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
            if not os.path.exists(dirname):
                os.mkdir(dirname)
            filename = os.path.join(dirname, token)
            fd = open(filename, 'w')
            upload = self.value
            fd.write(upload.fp.read())
            fd.close()
            if not session.tempfiles:
                session.tempfiles = {}
            session.tempfiles[token] = {
                'orig_filename': upload.orig_filename,
                'base_filename': upload.base_filename,
                'content_type': upload.content_type,
                'charset': upload.charset
            }
            self.value = None
        else:
            token = self.get('token')

        if session.tempfiles and session.tempfiles.has_key(token):
            self.token = token
            if not isinstance(self.value, PicklableUpload):
                self.value = PicklableUpload(session.tempfiles[token]['orig_filename'],
                    session.tempfiles[token]['content_type'],
                    session.tempfiles[token]['charset'])
                self.value.fp = tempfile.TemporaryFile()
                dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
                filename = os.path.join(dirname, token)
                self.value.fp.write(open(filename).read())
            self.value.fp.seek(0)

    def cleanup(self):
        if self.token:
            dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
            filename = os.path.join(dirname, self.token)
            if os.path.exists(filename):
                os.unlink(filename)

class PicklableUpload(Upload):
    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['fp']
        odict['data'] = self.fp.read()
        self.fp.seek(0)
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.fp = cStringIO.StringIO(self.data)
        del self.data

class EmailWidget(StringWidget):
    def __init__(self, *args, **kwargs):
        StringWidget.__init__(self, *args, **kwargs)
        self.attrs['size'] = '35'

    def _parse(self, request):
        StringWidget._parse(self, request)
        if self.value is not None:
            # basic tests first
            if not '@' in self.value[1:-1]:
                self.error = _('must be a valid email address')
            elif self.value[0] != '"' and ' ' in self.value:
                self.error = _('must be a valid email address')
            elif self.value[0] != '"' and self.value.count('@') != 1:
                self.error = _('must be a valid email address')
            elif get_cfg('emails', {}).get('check_domain_with_dns', True):
                # testing for domain existence
                domain = self.value.split('@')[-1]
                # simply lookup host name; note it will fail if hostname
                # doesn't have an A entry
                try:
                    socket.gethostbyname(domain)
                except socket.error:
                    if DNS is None or len(DNS.mxlookup(domain)) == 0:
                        self.error = _('invalid address domain')
                else:
                    # and then, localpart could be tested
                    pass


class WcsExtraStringWidget(StringWidget):
    field = None
    prefill = False

    def render_content(self):
        s = StringWidget.render_content(self)
        if not self.value and self.field and self.field.wsf_prefill_explicit and \
                get_session().lasso_session_dump:
            s += htmltext(
                    '<input class="prefill-button" type="submit" name="f%s-prefill" value="%s">') % (
                        self.field.id, _('Prefill'))
        return s

    def _parse(self, request):
        StringWidget._parse(self, request)
        if self.field and hasattr(self.field, 'prefill') and \
                self.field.prefill and self.field.prefill.get('type') == 'wsf':
            self.prefill = (request.get_field('f%s-prefill' % self.field.id) is not None)
            if self.prefill:
                if not request.form.has_key('prefill'):
                    request.form['prefill'] = []
                request.form['prefill'].append(self.field.id)
                self.error = 'MUST PREFILL...' # won't be displayed
        elif self.field and self.field.validation and self.value is not None:
            if not re.match(self.field.validation, self.value):
                self.error = _('wrong format')


class DateWidget(StringWidget):
    '''StringWidget which checks the value entered is a correct date'''
    minimum_date = None
    maximum_date = None

    def __init__(self, *args, **kwargs):
        # jquery datepicker default is to have the minimum pickable date to be
        # in the future, explicitely set a minimum date to get past that
        self.minimum_date = (1805, 9, 9)

        if kwargs.get('minimum_date'):
            self.minimum_date = time.strptime(kwargs.get('minimum_date'), misc.date_format())[:3]
            del kwargs['minimum_date']
        if kwargs.get('maximum_date'):
            self.maximum_date = time.strptime(kwargs.get('maximum_date'), misc.date_format())[:3]
            del kwargs['maximum_date']
        if kwargs.get('minimum_is_future'):
            if kwargs.get('date_can_be_today'):
                self.minimum_date = None
            else:
                self.minimum_date = (datetime.datetime.today() + datetime.timedelta(1)).timetuple()
        if kwargs.get('date_in_the_past'):
            if kwargs.get('date_can_be_today'):
                self.maximum_date = time.localtime()
            else:
                self.maximum_date = (datetime.datetime.today() - datetime.timedelta(1)).timetuple()

        if 'minimum_is_future' in kwargs:
            del kwargs['minimum_is_future']
        if 'date_in_the_past' in kwargs:
            del kwargs['date_in_the_past']
        if 'date_can_be_today' in kwargs:
            del kwargs['date_can_be_today']

        StringWidget.__init__(self, *args, **kwargs)
        self.attrs['size'] = '10'
        self.attrs['maxlength'] = '10'

    def _parse(self, request):
        StringWidget._parse(self, request)
        if self.value is not None:
            format_string = misc.date_format()
            try:
                value = time.strptime(self.value, format_string)
            except ValueError:
                self.error = _('invalid date')
                self.value = None
            else:
                if value[0] < 1800 or value[0] > 2099:
                    self.error = _('invalid date')
                    self.value = None
                elif self.minimum_date and value[:3] < self.minimum_date[:3]:
                    self.error = _('invalid date: date must be on or after %s') % time.strftime(
                            format_string, self.minimum_date)
                elif self.maximum_date and value[:3] > self.maximum_date[:3]:
                    self.error = _('invalid date; date must be on or before %s') % time.strftime(
                            format_string, self.maximum_date)

    def render_content(self):
        self.attrs['id'] = 'id_%s' % self.name
        self.attrs['class'] = 'date-pick'

        if self.attrs.get('readonly'):
            return StringWidget.render_content(self)

        get_response().add_javascript(['jquery.js',
                'bgiframe/jquery.bgiframe.min.js', 'methods/date.js'])
        current_language = get_request().language

        if current_language in ('fr', 'de'):
            get_response().add_javascript(['methods/date_%s.js' % current_language])

        date_format = misc.date_format().replace('%Y', 'yyyy').replace(
                '%m', 'mm').replace('%d', 'dd')
        get_response().add_javascript_code('Date.format = \'%s\';' % date_format)

        if current_language != 'en' and \
                               current_language in get_publisher().supported_languages or []:
            get_response().add_javascript_code('''\
$.dpText = {
  TEXT_PREV_YEAR   : '%(previous_year)s',
  TEXT_PREV_MONTH  : '%(previous_month)s',
  TEXT_NEXT_YEAR   : '%(next_year)s',
  TEXT_NEXT_MONTH  : '%(next_month)s',
  TEXT_CLOSE       : '%(close)s',
  TEXT_CHOOSE_DATE : '%(choose_date)s'
}
''' % { 'previous_year': _('Previous Year'),
        'previous_month': _('Previous Month'),
        'next_year': _('Next Year'),
        'next_month': _('Next Month'),
        'close': _('Close'),
        'choose_date': _('Choose Date')})

        get_response().add_javascript(['datePicker/jquery.datePicker.js'])
        get_response().add_css_include('datePicker.css')
        args = []
        if self.minimum_date:
            start_date = date_format.replace(
                    'yyyy', '%04d' % self.minimum_date[0]).replace(
                    'mm', '%02d' % self.minimum_date[1]).replace(
                    'dd', '%02d' % self.minimum_date[2])
            args.append('startDate:\'%s\'' % start_date)

        if self.maximum_date:
            end_date = date_format.replace(
                    'yyyy', '%04d' % self.maximum_date[0]).replace(
                    'mm', '%02d' % self.maximum_date[1]).replace(
                    'dd', '%02d' % self.maximum_date[2])
            args.append('endDate:\'%s\'' % end_date)

        if args:
            args = '{' + ','.join(args) + '}'
        else:
            args = ''

        get_response().add_javascript_code('''
$(function() {
        $('#%(id)s').datePicker(%(args)s).next().html(
            '<img src="%(root_url)sjs/calendar.png" width="16" height="16" />');
});''' % {'id': self.attrs['id'], 'args': args,
            'root_url': get_publisher().get_root_url() + get_publisher().qommon_static_dir})

        return StringWidget.render_content(self)


class RegexStringWidget(StringWidget):
    '''StringWidget which checks the value entered is a correct regex'''
    def _parse(self, request):
        StringWidget._parse(self, request)
        if self.value is not None:
            try:
                re.compile(self.value)
            except:
                self.error = _('invalid regular expression')
                self.value = None


class CheckboxesWidget(CompositeWidget):
    readonly = False

    def __init__(self, name, value = None, elements = None, **kwargs):
        CompositeWidget.__init__(self, name, value, **kwargs)
        self.element_names = {}

        if kwargs.has_key('title'):
            del kwargs['title']
        if kwargs.has_key('readonly'):
            del kwargs['readonly']
            self.readonly = True
        self.inline = kwargs.get('inline', True)
        self.max_choices = kwargs.get('max_choices', None)

        for v in elements:
            if type(v) is tuple:
                title = v[1]
                key = v[0]
                if type(key) is int:
                    name = 'element%d' % v[0]
                elif type(key) in (str, htmltext):
                    name = str('element%s' % v[0])
                    key = str(key)
                else:
                    raise NotImplementedError()
            else:
                title = v
                key = v
                name = 'element%d' % len(self.element_names.keys())

            if value and key in value:
                checked = True
            else:
                checked = False
            self.add(CheckboxWidget, name, title = title, value = checked, **kwargs)
            self.element_names[name] = key

    def _parse(self, request):
        if self.readonly:
            return
        values = []
        for name in self.element_names:
            value = self.get(name)
            if value is True:
                values.append(self.element_names[name])
        self.value = values

    def parse(self, request=None):
        CompositeWidget.parse(self, request=request)
        if request is None:
            request = get_request()
        if request.form or request.get_method() == 'POST':
            if self.required and not self.value:
                self.set_error(self.REQUIRED_ERROR)
            if self.value and self.max_choices and len(self.value) > self.max_choices:
                self.set_error(_('You must select at most %d answers.') % self.max_choices)
        return self.value

    def set_value(self, value):
        self.value = value
        for name in self.element_names:
            widget = self.get_widget(name)
            if self.element_names[name] in self.value:
                widget.set_value(True)
            else:
                widget.set_value(False)

    def has_error(self, request=None):
        return Widget.has_error(self, request=request)

    def render_content(self):
        r = TemplateIO(html=True)
        if self.inline:
            r += htmltext('<ul class="inline">')
        else:
            r += htmltext('<ul>')
        for widget in self.get_widgets():
            r += htmltext('<li><label>')
            if self.readonly:
                widget.attrs['disabled'] = 'disabled'
                if widget.value:
                    r += htmltext('<input type="hidden" name="%s" value="yes" >') % widget.name
                widget.name = widget.name + 'xx'
            r += widget.render_content()
            r += htmltext('</label>')
            r += widget.title
            r += htmltext('</li>')
        r += htmltext('</ul>')
        return r.getvalue()

class ValidatedStringWidget(StringWidget):
    '''StringWidget which checks the value entered is correct according to a regex'''
    regex = None

    def _parse(self, request):
        StringWidget._parse(self, request)
        if self.regex and self.value is not None:
            match = re.match(self.regex, self.value)
            if not match or not match.group() == self.value:
                self.error = _('wrong format')

class UrlWidget(ValidatedStringWidget):
    '''StringWidget which checks the value entered is a correct url starting with http or https'''
    regex = '^https?://.+'

    def _parse(self, request):
        ValidatedStringWidget._parse(self, request)
        if self.error:
            self.error = _('must start with http:// or https:// and have a domain name')

class ValidUrlWidget(UrlWidget):
    '''StringWidget which checks the value entered is a correct url for an existing page'''
    def _parse(self, request):
        UrlWidget._parse(self, request)
        # Default value that indicates the proxy (if configured) is not disabled for this host yet
        self.use_proxy = True
        if self.value and not self.error:
            status = 0
            try:
                response, status, page, auth_header = misc.http_get_page(self.value)
            except Exception, msg:
                # If the proxy is configured and the connection failed, try without the proxy
                if not get_cfg('proxy', {}).get('enabled'):
                    self.error = msg
                else:
                    try:
                        response, status, page, auth_header = misc.http_get_page(self.value, use_proxy=False)
                    except Exception, msg:
                        self.error = msg
                        return
                    else:
                        # Mark in the field that this host shouldn't use the proxy
                        self.use_proxy = False
            if status == 401:
                return
            elif status == 403:
                self.error = _('client error: access forbidden (error 403)')
            elif status == 404:
                self.error = _('client error: page not found (error 404)')
            elif str(status)[0] == '4':
                self.error = _('client error: %(reason)s (error %(code)s)') % {
                        'reason': status_reasons.get(status), 'code': status}
            elif str(status)[0] == '5':
                self.error = _('server error: %(reason)s (error %(code)s)') % {
                        'reason': status_reasons.get(status), 'code': status}

class UrlOrAbsPathWidget(UrlWidget):
    '''StringWidget which checks the value entered is a correct url for an existing page'''
    regex = '^https?://.+'

    def _parse(self, request):
        UrlWidget._parse(self, request)
        if self.error:
            self.error = None
            self.regex = '^/.*'
            UrlWidget._parse(self, request)
            if self.error:
                self.error = _('must start with http:// or https:// and have a domain name or start with /')


class CaptchaWidget(CompositeWidget):
    def __init__(self, name, value = None, mode = 'arithmetic', *args, **kwargs):
        CompositeWidget.__init__(self, name, value, **kwargs)
        if value:
            self.add(HiddenWidget, 'token', value.get('token'))
        else:
            token = get_session().create_captcha_token()
            self.add(HiddenWidget, 'token', token.get('token'))

        # create question, and fill token['answer']
        if mode == 'arithmetic':
            a, b = random.randint(2, 9), random.randint(2, 9)
            while b == a:
                # don't get twice the same number
                b = random.randint(2, 9)
            operator = random.choice([N_('times'), N_('plus'), N_('minus')])
            if operator == 'times':
                answer = a * b
            elif operator == 'plus':
                answer = a + b
            elif operator == 'minus':
                if b > a:
                    a, b = b, a
                answer = a - b
            question = _('What is the result of %(a)d %(op)s %(b)d?') % {
                    'a': a, 'b': b, 'op': _(operator)}
            hint = _('Please answer this simple mathematical question as proof you are not a bot.')
        self.add(StringWidget, 'q', title = question, hint = hint)
        token['answer'] = str(answer)

    def _parse(self, request):
        v = {'answer': self.get('q'),
             'token': self.get('token')}
        token = get_session().get_captcha_token(self.get('token'))
        if token['answer'] != v['answer']:
            self.error = _('wrong answer')
        else:
            get_session().won_captcha = True
            self.value = v


class WidgetList(quixote.form.widget.WidgetList):
    def render(self):
        get_response().add_javascript(['jquery.js', 'widget_list.js'])
        r = TemplateIO(html=True)
        r += self.render_title(self.get_title())
        r += self.render_error(self.get_error())
        add_element_widget = self.get_widget('add_element')
        for widget in self.get_widgets():
            if widget is add_element_widget:
                continue
            r += widget.render()
        r += add_element_widget.render()
        r += self.render_hint(self.get_hint())
        return r.getvalue()

class WidgetDict(quixote.form.widget.WidgetDict):
    # Fix the title and hint setting
    # FIXME: to be fixed in Quixote upstream : title and hint parameters should be removed
    def __init__(self, name, value=None, title='', hint='',
                 element_key_type=StringWidget,
                 element_value_type=StringWidget,
                 element_key_kwargs={},
                 element_value_kwargs={},
                 add_element_label='Add row', **kwargs):

        quixote.form.widget.WidgetDict.__init__(self, name, value, **kwargs)
        if title:
            self.title = title
        if hint:
            self.hint = hint

class TagsWidget(StringWidget):
    def __init__(self, name, value = None, known_tags = None, **kwargs):
        StringWidget.__init__(self, name, value, **kwargs)
        self.known_tags = known_tags

    def _parse(self, request):
        StringWidget._parse(self, request)
        if self.value is not None:
            self.value = [x.strip() for x in self.value.split(',') if x.strip()]

    def render_content(self):
        r = TemplateIO(html=True)
        get_response().add_javascript(['jquery.js', 'jquery.autocomplete.js'])
        get_response().add_css_include('../js/jquery.autocomplete.css')
        id = 'tags-%s' % randbytes(8)
        if self.value:
            value = ', '.join(self.value) + ', '
        else:
            value = ''
        r += htmltag('input', xml_end = True,
                id = id,
                type = self.HTML_TYPE,
                name = self.name,
                value = value,
                **self.attrs)
        if self.known_tags:
            known_tags = ', '.join([repr(x) for x in sorted(self.known_tags)])
        else:
            known_tags = ''
        r += htmltext('''<script type="text/javascript">
$("#%s").autocompleteArray([
    %s
    ],{mode:'multiple', multipleSeparator:', '});
</script>''' % (id, known_tags))
        return r.getvalue()

class WysiwygTextWidget(TextWidget):
    def _parse(self, request):
        TextWidget._parse(self, request)
        if self.value:
            if _sanitizeHTML:
                self.value = _sanitizeHTML(self.value, get_request().charset, 'text/html')
            else:
                self.value = htmlescape(self.value)
            if self.value.startswith('<br />'):
                self.value = self.value[6:]
            if self.value.endswith('<br />'):
                self.value = self.value[:-6]

    def render_content(self):
        get_response().add_javascript(['jquery.js', 'jwysiwyg/jquery.wysiwyg.js'])
        get_response().add_css_include('../js/jwysiwyg/jquery.wysiwyg.css')
        # TODO: allow this to be specified

        insertUl = str(self.attrs.get('insertUl', True)).lower()
        insertLink = str(self.attrs.get('insertLink', False)).lower()
        insertImg = str(self.attrs.get('insertImg', False)).lower()

        attrs = self.attrs.copy()
        if attrs.has_key('insertUl'):
            del attrs['insertUl']
        if attrs.has_key('insertLink'):
            del attrs['insertLink']
        if attrs.has_key('insertImg'):
            del attrs['insertImg']

        className = self.__class__.__name__
        name = self.name
        get_response().add_javascript_code('''$(function()
{
    $('div.%(className)s textarea#wysiwyg-%(name)s').wysiwyg({
        controls : {
            insertUnorderedList : {visible: %(insertUl)s},
            separator04 : {visible: %(insertUl)s},
            h1mozilla : {visible: false},
            h2mozilla : {visible: false},
            h3mozilla : {visible: false},
            h1 : {visible: false},
            h2 : {visible: false},
            h3 : {visible: false},
            separator08: {visible: false},
            separator09: {visible: false},
            increaseFontSize: {visible: false},
            decreaseFontSize: {visible: false},
            createLink: {visible: %(insertLink)s},
            insertImage: {visible: %(insertImg)s},
            separator06: {visible: false}
        }});
});''' % locals())
        return (htmltag('textarea', name=self.name, id='wysiwyg-' + self.name, **attrs) +
                htmlescape(self.value or '') +
                htmltext("</textarea>"))


class TableWidget(CompositeWidget):
    readonly = False

    def __init__(self, name, value = None, rows = None, columns = None, **kwargs):
        CompositeWidget.__init__(self, name, value, **kwargs)
        self.rows = rows
        self.columns = columns

        widget_kwargs = {}
        if kwargs.has_key('title'):
            del kwargs['title']
        if kwargs.has_key('readonly') and kwargs.get('readonly'):
            del kwargs['readonly']
            self.readonly = True
            widget_kwargs['readonly'] = 'readonly'

        for i, row in enumerate(rows):
            for j, column in enumerate(columns):
                widget = self.add(StringWidget, 'c-%s-%s' % (i, j), **widget_kwargs)
                widget = self.get_widget('c-%s-%s' % (i, j))
                if value and self.readonly:
                    if not get_request().form:
                        get_request().form = {}
                    get_request().form[widget.name] = value[i][j]
                    widget.set_value(value[i][j])

    def render_content(self):
        r = TemplateIO(html=True)
        r += htmltext('<table><thead><tr><td></td>')
        for column in self.columns:
            r += htmltext('<th>%s</th>') % column
        r += htmltext('</tr></thead><tbody>')
        for i, row in enumerate(self.rows):
            r += htmltext('<tr><th>%s</th>') % row
            for j, column in enumerate(self.columns):
                widget = self.get_widget('c-%s-%s' % (i, j))
                r += htmltext('<td>')
                r += widget.render_content()
                r += htmltext('</td>')
            r += htmltext('</tr>')
        r += htmltext('</tbody></table>')
        return r.getvalue()

    def parse(self, request=None):
        CompositeWidget.parse(self, request=request)
        if request is None:
            request = get_request()
        if request.form or request.get_method() == 'POST':
            if self.required and not self.value:
                self.set_error(self.REQUIRED_ERROR)
        return self.value

    def _parse(self, request):
        if self.readonly:
            return
        table = []
        for i, row in enumerate(self.rows):
            row = []
            for j, column in enumerate(self.columns):
                widget = self.get_widget('c-%s-%s' % (i, j))
                row.append(widget.parse())
            table.append(row)
        self.value = table

    def set_value(self, value):
        self.value = value
        if not value:
            return
        for i, row in enumerate(self.rows):
            for j, column in enumerate(self.columns):
                widget = self.get_widget('c-%s-%s' % (i, j))
                widget.set_value(value[i][j])

