try:
    from hashlib import sha1 as sha
except ImportError:
    from sha import sha

from quixote import get_publisher

from qommon.storage import StorableObject

class PasswordAccount(StorableObject):
    _names = 'passwordaccounts'
    _indexes = ['user_id']

    id = None  # id is username
    password = None
    hashing_algo = None

    awaiting_confirmation = False
    awaiting_moderation = False
    disabled = False

    warned_about_unused_account = False

    user_id = None

    def get_with_credentials(cls, username, password):
        account = cls.get(username)
        if not account.is_password_ok(password):
            raise KeyError()
        return get_publisher().user_class.get(account.user_id)
    get_with_credentials = classmethod(get_with_credentials)

    def get_user(self):
        try:
            return get_publisher().user_class.get(self.user_id)
        except KeyError:
            return None
    user = property(get_user)

    def is_password_ok(self, password):
        if self.hashing_algo is None:
            return (self.password == password)
        elif self.hashing_algo == 'sha':
            return (self.password == sha(password).hexdigest())

    def set_password(self, password):
        if self.hashing_algo is None:
            self.password = password
        elif self.hashing_algo == 'sha':
            self.password = sha(password).hexdigest()

