import logging
import os

from quixote import get_publisher, get_session, get_request


class BotFilter(logging.Filter):
    def filter(self, record):
        if record.levelno >= logging.ERROR:
            # always log errors
            return 1
        if self.is_bot():
            return 0
        return 1

    def is_bot(cls):
        botfile = os.path.join(get_publisher().data_dir, 'webbots')
        if os.path.exists(botfile):
            user_agent = get_request().get_environ('HTTP_USER_AGENT', '')
            for bot_ua_string in [x.strip() for x in open(botfile).readlines()]:
                if bot_ua_string in user_agent:
                    return True
        return False
    is_bot = classmethod(is_bot)


class Formatter(logging.Formatter):
    def format(self, record):
        request = get_request()

        record.address = request.get_environ('REMOTE_ADDR', '-')
        record.path = request.get_path()
        record.session_id = get_session().get_session_id() or '[nosession]'

        user = request.user
        if user:
            if type(user) in (str, int, unicode):
                user_id = user
            else:
                user_id = user.id
                if type(user_id) is str and user_id.startswith('anonymous-'):
                    user_id = 'anonymous'
        else:
            user_id = 'unlogged'
            if BotFilter.is_bot():
                user_id = 'bot'
        record.user_id = user_id

        return logging.Formatter.format(self, record)


def readline(line):
    try:
        date, hour, level, ip, session_id, url, user_id, dash, message = line.strip().split(' ', 8)
    except ValueError:
        date, hour, level, message = line.strip().split(' ', 3)
        return locals()
    if dash != '-':
        return None
    return locals()

