import cPickle
import re
import os
import time
import httplib
import urllib
import socket
import base64

from quixote import get_publisher, get_session
from quixote.html import htmltext

from qommon import get_cfg, get_logger
from qommon.errors import ConnectionError

import strftime

def get_abs_path(s):
    if not s:
        return s
    if s[0] == '/':
        return s
    return os.path.join(get_publisher().app_dir, s)

def get_lasso_server(protocol = 'liberty'):
    if not get_cfg('sp'):
        return None
    import lasso
    if protocol == 'liberty':
        server = lasso.Server(
            get_abs_path(get_cfg('sp')['metadata']),
            get_abs_path(get_cfg('sp')['privatekey']),
            None, None)
    elif protocol == 'saml2' and get_cfg('sp').has_key('saml2_metadata'):
        server = lasso.Server(
            get_abs_path(get_cfg('sp')['saml2_metadata']),
            get_abs_path(get_cfg('sp')['privatekey']),
            None, None)
    else:
        raise 'XXX: unknown protocol'

    # Set encryption private key
    if protocol in ('liberty', 'saml2'):
        encryption_privatekey = get_abs_path(get_cfg('sp').get('encryption_privatekey'))
        if encryption_privatekey and os.path.exists(encryption_privatekey):
            try:
                server.setEncryptionPrivateKey(encryption_privatekey)
            except lasso.Error, error:
                get_logger().warn('Failed to set encryption private key')

    for klp, idp in get_cfg('idp', {}).items():
        try:
            server.addProvider(
                    lasso.PROVIDER_ROLE_IDP,
                    get_abs_path(idp['metadata']),
                    get_abs_path(idp.get('publickey')),
                    get_abs_path(idp.get('cacertchain')))
        except lasso.Error, error:
            if error[0] == lasso.SERVER_ERROR_ADD_PROVIDER_PROTOCOL_MISMATCH:
                continue
            if error[0] == lasso.SERVER_ERROR_ADD_PROVIDER_FAILED:
                continue
            raise

        if hasattr(lasso, 'ENCRYPTION_SYM_KEY_TYPE_DEFAULT'):
            encryption_mode = lasso.ENCRYPTION_MODE_NONE
            if idp.get('encrypt_nameid', False):
                encryption_mode |= lasso.ENCRYPTION_MODE_NAMEID
            provider_t = get_provider(klp)
            provider = server.getProvider(provider_t.providerId)
            if provider is not None:
                provider.setEncryptionMode(encryption_mode);

    return server

def get_provider_label(provider):
    if not provider:
        return None
    if not hasattr(provider, str('getOrganization')):
        return provider.providerId

    organization = provider.getOrganization()
    if not organization:
        return provider.providerId

    name = re.findall("<OrganizationDisplayName.*>(.*?)</OrganizationDisplayName>", organization)
    if not name:
        name = re.findall("<OrganizationName.*>(.*?)</OrganizationName>", organization)
        if not name:
            return provider.providerId
    return htmltext(name[0])

def get_provider(provider_key):
    lp = get_cfg('idp', {}).get(provider_key)
    if not lp:
        raise KeyError()

    import lasso

    publickey_fn = None
    if lp.get('publickey'):
        publickey_fn = get_abs_path(lp['publickey'])
    # cacertchain (not really necessary to get provider label)

    try:
        provider = lasso.Provider(lasso.PROVIDER_ROLE_IDP,
                get_abs_path(lp['metadata']), publickey_fn, None)
    except lasso.Error:
        raise KeyError()

    return provider

def get_provider_key(provider_id):
    return provider_id.replace('://', '-').replace('/', '-').replace('?', '-').replace(':', '-')

def get_current_protocol():
    if not get_session().lasso_identity_provider_id:
        return None

    provider = get_provider(get_provider_key(get_session().lasso_identity_provider_id))
    return provider.getProtocolConformance()


xlate = {
    u'\N{ACUTE ACCENT}': "'",
    u'\N{BROKEN BAR}': '|',
    u'\N{CEDILLA}': '{cedilla}',
    u'\N{CENT SIGN}': '{cent}',
    u'\N{COPYRIGHT SIGN}': '{C}',
    u'\N{CURRENCY SIGN}': '{currency}',
    u'\N{DEGREE SIGN}': '{degrees}',
    u'\N{DIAERESIS}': '{umlaut}',
    u'\N{DIVISION SIGN}': '/',
    u'\N{FEMININE ORDINAL INDICATOR}': '{^a}',
    u'\N{INVERTED EXCLAMATION MARK}': '!',
    u'\N{INVERTED QUESTION MARK}': '?',
    u'\N{LATIN CAPITAL LETTER A WITH ACUTE}': 'A',
    u'\N{LATIN CAPITAL LETTER A WITH CIRCUMFLEX}': 'A',
    u'\N{LATIN CAPITAL LETTER A WITH DIAERESIS}': 'A',
    u'\N{LATIN CAPITAL LETTER A WITH GRAVE}': 'A',
    u'\N{LATIN CAPITAL LETTER A WITH RING ABOVE}': 'A',
    u'\N{LATIN CAPITAL LETTER A WITH TILDE}': 'A',
    u'\N{LATIN CAPITAL LETTER AE}': 'Ae',
    u'\N{LATIN CAPITAL LETTER C WITH CEDILLA}': 'C',
    u'\N{LATIN CAPITAL LETTER E WITH ACUTE}': 'E',
    u'\N{LATIN CAPITAL LETTER E WITH CIRCUMFLEX}': 'E',
    u'\N{LATIN CAPITAL LETTER E WITH DIAERESIS}': 'E',
    u'\N{LATIN CAPITAL LETTER E WITH GRAVE}': 'E',
    u'\N{LATIN CAPITAL LETTER ETH}': 'Th',
    u'\N{LATIN CAPITAL LETTER I WITH ACUTE}': 'I',
    u'\N{LATIN CAPITAL LETTER I WITH CIRCUMFLEX}': 'I',
    u'\N{LATIN CAPITAL LETTER I WITH DIAERESIS}': 'I',
    u'\N{LATIN CAPITAL LETTER I WITH GRAVE}': 'I',
    u'\N{LATIN CAPITAL LETTER N WITH TILDE}': 'N',
    u'\N{LATIN CAPITAL LETTER O WITH ACUTE}': 'O',
    u'\N{LATIN CAPITAL LETTER O WITH CIRCUMFLEX}': 'O',
    u'\N{LATIN CAPITAL LETTER O WITH DIAERESIS}': 'O',
    u'\N{LATIN CAPITAL LETTER O WITH GRAVE}': 'O',
    u'\N{LATIN CAPITAL LETTER O WITH STROKE}': 'O',
    u'\N{LATIN CAPITAL LETTER O WITH TILDE}': 'O',
    u'\N{LATIN CAPITAL LETTER THORN}': 'th',
    u'\N{LATIN CAPITAL LETTER U WITH ACUTE}': 'U',
    u'\N{LATIN CAPITAL LETTER U WITH CIRCUMFLEX}': 'U',
    u'\N{LATIN CAPITAL LETTER U WITH DIAERESIS}': 'U',
    u'\N{LATIN CAPITAL LETTER U WITH GRAVE}': 'U',
    u'\N{LATIN CAPITAL LETTER Y WITH ACUTE}': 'Y',
    u'\N{LATIN SMALL LETTER A WITH ACUTE}': 'a',
    u'\N{LATIN SMALL LETTER A WITH CIRCUMFLEX}': 'a',
    u'\N{LATIN SMALL LETTER A WITH DIAERESIS}': 'a',
    u'\N{LATIN SMALL LETTER A WITH GRAVE}': 'a',
    u'\N{LATIN SMALL LETTER A WITH RING ABOVE}': 'a',
    u'\N{LATIN SMALL LETTER A WITH TILDE}': 'a',
    u'\N{LATIN SMALL LETTER AE}': 'ae',
    u'\N{LATIN SMALL LETTER C WITH CEDILLA}': 'c',
    u'\N{LATIN SMALL LETTER E WITH ACUTE}': 'e',
    u'\N{LATIN SMALL LETTER E WITH CIRCUMFLEX}': 'e',
    u'\N{LATIN SMALL LETTER E WITH DIAERESIS}': 'e',
    u'\N{LATIN SMALL LETTER E WITH GRAVE}': 'e',
    u'\N{LATIN SMALL LETTER ETH}': 'th',
    u'\N{LATIN SMALL LETTER I WITH ACUTE}': 'i',
    u'\N{LATIN SMALL LETTER I WITH CIRCUMFLEX}': 'i',
    u'\N{LATIN SMALL LETTER I WITH DIAERESIS}': 'i',
    u'\N{LATIN SMALL LETTER I WITH GRAVE}': 'i',
    u'\N{LATIN SMALL LETTER N WITH TILDE}': 'n',
    u'\N{LATIN SMALL LETTER O WITH ACUTE}': 'o',
    u'\N{LATIN SMALL LETTER O WITH CIRCUMFLEX}': 'o',
    u'\N{LATIN SMALL LETTER O WITH DIAERESIS}': 'o',
    u'\N{LATIN SMALL LETTER O WITH GRAVE}': 'o',
    u'\N{LATIN SMALL LETTER O WITH STROKE}': 'o',
    u'\N{LATIN SMALL LETTER O WITH TILDE}': 'o',
    u'\N{LATIN SMALL LETTER SHARP S}': 'ss',
    u'\N{LATIN SMALL LETTER THORN}': 'th',
    u'\N{LATIN SMALL LETTER U WITH ACUTE}': 'u',
    u'\N{LATIN SMALL LETTER U WITH CIRCUMFLEX}': 'u',
    u'\N{LATIN SMALL LETTER U WITH DIAERESIS}': 'u',
    u'\N{LATIN SMALL LETTER U WITH GRAVE}': 'u',
    u'\N{LATIN SMALL LETTER Y WITH ACUTE}': 'y',
    u'\N{LATIN SMALL LETTER Y WITH DIAERESIS}': 'y',
    u'\N{LEFT-POINTING DOUBLE ANGLE QUOTATION MARK}': '<<',
    u'\N{MACRON}': '_',
    u'\N{MASCULINE ORDINAL INDICATOR}': '{^o}',
    u'\N{MICRO SIGN}': '{micro}',
    u'\N{MIDDLE DOT}': '*',
    u'\N{MULTIPLICATION SIGN}': '*',
    u'\N{NOT SIGN}': '{not}',
    u'\N{PILCROW SIGN}': '{paragraph}',
    u'\N{PLUS-MINUS SIGN}': '{+/-}',
    u'\N{POUND SIGN}': '{pound}',
    u'\N{REGISTERED SIGN}': '{R}',
    u'\N{RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK}': '>>',
    u'\N{SECTION SIGN}': '{section}',
    u'\N{SOFT HYPHEN}': '-',
    u'\N{SUPERSCRIPT ONE}': '{^1}',
    u'\N{SUPERSCRIPT THREE}': '{^3}',
    u'\N{SUPERSCRIPT TWO}': '{^2}',
    u'\N{VULGAR FRACTION ONE HALF}': '{1/2}',
    u'\N{VULGAR FRACTION ONE QUARTER}': '{1/4}',
    u'\N{VULGAR FRACTION THREE QUARTERS}': '{3/4}',
    u'\N{YEN SIGN}': '{yen}'
    }

def latin1_to_ascii (unicrap):
    """This takes a UNICODE string and replaces Latin-1 characters with
        something equivalent in 7-bit ASCII. It returns a plain ASCII string.
        This function makes a best effort to convert Latin-1 characters into
        ASCII equivalents. It does not just strip out the Latin-1 characters.
        All characters in the standard 7-bit ASCII range are preserved.
        In the 8th bit range all the Latin-1 accented letters are converted
        to unaccented equivalents. Most symbol characters are converted to
        something meaningful. Anything not converted is deleted.

        <http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/251871>
    """

    if not isinstance(unicrap, unicode):
        if get_publisher() and get_publisher().site_charset:
            unicrap = unicode(unicrap, get_publisher().site_charset, 'ignore')
        else:
            unicrap = unicode(unicrap, 'iso-8859-1', 'ignore')
    r = ""
    for i in unicrap:
        if xlate.has_key(i):
            r += xlate[i]
        elif ord(i) >= 0x80:
            pass
        else:
            r += str(i)
    return r

simplifyRegex = re.compile(r'''([\s()'"/:\'\.,\?\&\[\]]+)''')

def simplify(s, space = '-'):
    if s is None: s = ''
    s = latin1_to_ascii(simplifyRegex.sub(' ', s).strip()).replace(' ', space).lower()
    return re.sub(r'\%s+' % space, space, s) # remove consecutive dashes (or whatever)


def get_datetime_language():
    lang = get_cfg('language', {}).get('language', None)
    if lang is None:
        if os.environ.get('LC_TIME'):
            lang = os.environ.get('LC_TIME')[:2]
        elif os.environ.get('LC_ALL'):
            lang = os.environ.get('LC_ALL')[:2]
    return lang

def localstrftime(t):
    if not t:
        return ''
    format_string = '%Y-%m-%d %H:%M'
    lang = get_datetime_language()
    if lang == 'fr':
        format_string = '%d/%m/%Y %H:%M'
    return strftime.strftime(format_string, t)

def date_format():
    format_string = '%Y-%m-%d'
    lang = get_datetime_language()
    if lang == 'fr':
        format_string = '%d/%m/%Y'
    return format_string

def ellipsize(s, length = 30):
    if not s or len(s) < length:
        return s
    return s[:length-5] + ' (...)'

def get_month_name(month):
    month_names = [_('January'), _('February'), _('March'), _('April'),
            _('May'), _('June'), _('July'), _('August'),
            _('September'), _('October'), _('November'), _('December') ]
    return month_names[month-1]

def format_time(datetime, formatstring, gmtime = False):
    if not datetime:
        return '?'
    if type(datetime) in (int, float):
        if gmtime:
            datetime = time.gmtime(datetime)
        else:
            datetime = time.localtime(datetime)
    if len(datetime) == 2:
        year, month = datetime
        weekday = None
    elif len(datetime) == 3:
        year, month, day = datetime
        weekday = None
    else:
        year, month, day, hour, minute, second, weekday = datetime[:7]

    weekday_names = [_('Monday'), _('Tuesday'), _('Wednesday'),
            _('Thursday'), _('Friday'), _('Saturday'), _('Sunday')]

    if weekday is not None:
        weekday_name = weekday_names[weekday]
        lower_weekday_name = weekday_name.lower()
        abbr_weekday_name = weekday_name[:3]

    month_name = get_month_name(month)
    lower_month_name = month_name.lower()
    abbr_month_name = month_name[:3]

    return formatstring % locals()

def _http_request(url, method='GET', body=None, headers={}, use_proxy=True):
    get_publisher().reload_cfg()
    socket.setdefaulttimeout(3)

    if url.startswith('http://'):
        hostname, query = urllib.splithost(url[5:])
    else:
        hostname, query = urllib.splithost(url[6:])

    proxy = False
    proxy_cfg = get_cfg('proxy', {})
    if use_proxy is True and proxy_cfg.get('enabled'):
        proxy_ip = proxy_cfg.get('ip')
        proxy_port = proxy_cfg.get('port')
        proxy_user = proxy_cfg.get('user')
        proxy_password = proxy_cfg.get('password')
        if proxy_ip and proxy_port:
            proxy = True
            conn = httplib.HTTPConnection('%s:%s' % (proxy_ip, proxy_port))
            query = url
            if proxy_user and proxy_password:
                credentials = base64.encodestring('%s:%s' % (proxy_user, proxy_password))[:-1]
                auth = 'Basic %s' % credentials
                headers['Proxy-Authorization'] = auth
                headers['Proxy-Connection'] = 'keep-alive'
                headers['Keep-Alive'] = '300'

    if not proxy:
        if url.startswith('http://'):
            conn = httplib.HTTPConnection(hostname)
        else:
            conn = httplib.HTTPSConnection(hostname)
        query = query.replace('&amp;', '&')

    try:
        conn.request(method, query, body, headers)
    except socket.gaierror, (err, msg):
        conn.close()
        raise ConnectionError('error while connecting to %s : %s' % (hostname, msg))
    except socket.timeout, err:
        conn.close()
        raise ConnectionError('connection timed out while fetching the page')
    except socket.error, err:
        conn.close()
        raise ConnectionError('error while fetching the page : %s' % err)
    else:
        response = conn.getresponse()
        data = response.read()
        status = response.status
        auth_header = response.getheader('WWW-Authenticate')
        conn.close()

    return response, status, data, auth_header

def http_get_page(url, headers={}, use_proxy=True):
    return _http_request(url, headers=headers, use_proxy=use_proxy)

def http_post_request(url, body=None, headers={}, use_proxy=True):
    return _http_request(url, 'POST', body, headers, use_proxy=use_proxy)

