import time

from quixote import get_publisher

from quixote.session import Session as QuixoteSession
from quixote.session import SessionManager as QuixoteSessionManager
from quixote.util import randbytes

from qommon.storage import StorableObject

class Session(QuixoteSession, StorableObject):
    _names = 'sessions'
    MAX_CAPTCHA_TOKENS = 8

    name_identifier = None
    after_url = None
    lasso_session_dump = None
    lasso_anonymous_identity_dump = None
    lasso_identity_provider_id = None
    lasso_manage_name_id_dump = None
    message = None
    saml_idp_cookie = None
    lasso_session_index = None
    ident_idp_token = None
    _captcha_tokens = None
    won_captcha = False
    tempfiles = None

    username = None # only set on password authentication

    def has_info(self):
        return self.name_identifier or self.after_url or \
            self.lasso_session_dump or self.message or \
            self.lasso_anonymous_identity_dump or \
            self.lasso_manage_name_id_dump or \
            self.lasso_identity_provider_id or \
            self._captcha_tokens or \
            self.won_captcha or \
            self.ident_idp_token or \
            self.tempfiles or \
            QuixoteSession.has_info(self)
    is_dirty = has_info

    def get_session_id(self):
        return self.id

    def set_session_id(self, session_id):
        self.id = session_id

    session_id = property(get_session_id, set_session_id)

    def get_user(self):
        user_id = QuixoteSession.get_user(self)
        if user_id:
            if str(user_id).startswith('anonymous-'):
                user = get_publisher().user_class()
                user.id = user_id
                user.anonymous = True
                user.name_identifiers = [ self.name_identifier ]
                user.lasso_dump = self.lasso_anonymous_identity_dump
                return user
            else:
                try:
                    return get_publisher().user_class.get(user_id)
                except KeyError:
                    pass
        return None

    def set_user(self, user_id):
        QuixoteSession.set_user(self, user_id)
        try:
            user = get_publisher().user_class.get(user_id)
            user.last_seen = time.time()
            user.store()
        except KeyError:
            pass

    def display_message(self):
        if not self.message or not self.message[1]:
            return ''
        from quixote.html import htmltext
        s = htmltext('<div class="%snotice">%s</div>' % self.message)
        self.message = None
        return s

    def get_user_object(self):
        return self.get_user()

    def create_captcha_token(self):
        if not self._captcha_tokens:
            self._captcha_tokens = []
        token = {'token': randbytes(8), 'answer': None}
        self._captcha_tokens.append(token)
        extra = len(self._captcha_tokens) - self.MAX_CAPTCHA_TOKENS
        if extra > 0:
            del self._captcha_tokens[:extra]
        return token

    def get_captcha_token(self, token):
        if not self._captcha_tokens:
            return None
        try:
            return [x for x in self._captcha_tokens if x.get('token') == token][0]
        except IndexError:
            return None

    def remove_captcha_token(self, token):
        token = self.get_captcha_token(token)
        if token:
            self._captcha_tokens.remove(token)


class StorageSessionManager(QuixoteSessionManager):
    def __init__(self):
        QuixoteSessionManager.__init__(self, session_class=BasicSession)

    def forget_changes(self, session):
        pass

    def __getitem__(self, session_id):
        try:
            return BasicSession.get(session_id)
        except KeyError:
            raise KeyError

    def get(self, session_id, default = None):
        try:
            return BasicSession.get(session_id)
        except KeyError:
            return default

    def commit_changes(self, session):
        if session and session.id:
            session.store()

    def keys(self):
        return BasicSession.keys()

    def values(self):
        return BasicSession.values()

    def items(self):
        return BasicSession.items()

    def has_key(self, session_id):
        return BasicSession.has_key(session_id)

    def __setitem__(self, session_id, session):
        session.store()

    def __delitem__(self, session_id):
        if not session_id:
            return
        try:
            self.session_class.remove_object(session_id)
        except OSError:
            raise KeyError

